#!/usr/local/bin/perl

# @author Dilip Antony Joseph (dilip@cs.berekeley.edu)

use strict;
require Utils;

my $VERSION = "0.1";

my $TEST_CASE_DIR="cases";
my $BINARY_NAME = "KeywordHunter";
my $DUMP_DIR="~";
my $ERROR_EXIT_CODE=256;
my $SUCCESS_EXIT_CODE=0;

if ( ! -f $BINARY_NAME) {
  die "Unable to find the binary file $BINARY_NAME in the current directory.\n";
}

print "\n\n";
print "Project 2 Evaluator version $VERSION\n";

my $MaxPoints = 0;
my $TotalPoints = 0;

my $lTmpTotalPoints;
my $lTmpMaxPoints;


($lTmpTotalPoints, $lTmpMaxPoints) = Test_CmdLineArgs();
$TotalPoints += $lTmpTotalPoints;
$MaxPoints += $lTmpMaxPoints;

# Run the test cases
my @lCaseFiles = <$TEST_CASE_DIR/CASE*.txt>;
my $lCaseFile;
foreach $lCaseFile (@lCaseFiles) {
  ($lTmpTotalPoints, $lTmpMaxPoints) = RunCase($lCaseFile);
  $TotalPoints += $lTmpTotalPoints;
  $MaxPoints += $lTmpMaxPoints;
}



print "\n\n";
print "#######################################################\n";
print "Grand Total : $TotalPoints (out of $MaxPoints)\n\n";




sub Test_CmdLineArgs() {
  my $lTestName = "CmdLineArgs";

  my $lDumpFile = "$DUMP_DIR/proj2_dump.txt";

  my $lMaxPoints = 2;
  my $lPoints = 0;

  `./$BINARY_NAME > $lDumpFile`;
  if ($? == $ERROR_EXIT_CODE) {
    $lPoints = $lPoints + 1;
  }
 
  system("./$BINARY_NAME http://danjo.cs.berkeley.edu/index.html > $lDumpFile");
  if ($? == $ERROR_EXIT_CODE) {
    $lPoints ++;
  }
 
  if ($lPoints > $lMaxPoints) {
    die "Max Points exceeded for $lTestName\n";
  }

  print "TestResult: CmdLineArgs: $lPoints (out of $lMaxPoints)\n";

  return ($lPoints, $lMaxPoints);
 
}


sub RunCase() {
  my $lCaseFile = shift(@_);
  print "\n\n";
  print "Running Case : $lCaseFile\n";

  my $lPoints = 0;

  my $lCaseAnswerFile = "$lCaseFile.answer";
  if (!-f $lCaseAnswerFile) {
    die "Unable to find the answer file $lCaseAnswerFile.\n";
  }

  my $lCaseTmpFile = "$lCaseFile.out";
  if (-f $lCaseTmpFile) {
    unlink($lCaseTmpFile) || die "Unable to delete existing case output file $lCaseTmpFile\n";
  }

  #Extract the details of the case.
  my %lCaseDetails = readConfigFile ($lCaseFile);
  my $lStartUrl = $lCaseDetails{"StartUrl"};
  my $lLevel = $lCaseDetails{"Level"};
  my $lSearchKeyword = $lCaseDetails{"SearchKeyword"};
  my $lDesc = $lCaseDetails{"Description"};
  my $lMaxPoints = $lCaseDetails{"MaxPoints"};

  if (  !defined($lStartUrl) ||
        !defined($lLevel) ||
        !defined($lSearchKeyword) ||
        !defined($lMaxPoints) 
     ) {
    die "Invalid case file $lCaseFile.\n";
  }
  
  if (defined($lDesc)) {
    print "Description: $lDesc\n\n";
  }

  my $lPoints_Found = 1 * $lMaxPoints;

  my $lTmpCmd = "./$BINARY_NAME $lStartUrl $lSearchKeyword > $lCaseTmpFile";
  `$lTmpCmd`;
  if ($? != $SUCCESS_EXIT_CODE) {
    print "FAIL: $lTmpCmd returned exit code $?, expected $SUCCESS_EXIT_CODE\n";
  } else {
    if (CheckFoundLine($lCaseTmpFile, $lCaseAnswerFile) == 1) {
      $lPoints += $lPoints_Found;
    }
  }


  if ($lPoints > $lMaxPoints) {
    die "Max Points exceeded for Case $lCaseFile\n";
  }

  print "TestResult: $lCaseFile: $lPoints (out of $lMaxPoints)\n";

  return ($lPoints, $lMaxPoints);
}

sub CheckFoundLine() {
  my $lTestFile = shift(@_);
  my $lAnsFile = shift(@_);

  my $lAnsLine;

  open (AnsFd, "$lAnsFile") || die "Unable to open $lAnsFile for reading.\n";
  while(<AnsFd>) {
    if (/^FOUND_KEYWORD:/) {
      $lAnsLine = $_;
      last;
    } elsif (/^NOT_FOUND_KEYWORD:/) {
      $lAnsLine = $_;
    }
  }
  if (!defined($lAnsLine)) {
    die "CheckFoundLine: Invalid answer file!\n";
  }
  close (AnsFd);

  my $lFoundFlag = 0;
  open (TestFd, "$lTestFile") || die "Unable to open $lTestFile for reading.\n";
  while(<TestFd>) {
    if(/^$lAnsLine/) {
      $lFoundFlag = 1;
      last;
    }
  }
  close (TestFd);

  return $lFoundFlag;
  
}
